/* Copyright (C) 2017 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of simmc-agent
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include <agent/linux/parse_proxy_file.h>
#include <iostream>
#include <map>
#include <string>
/*---------------------------------------------------------------------
 * set Functions
 */

void Proxy::setHost(std::string h) {
    this->host = h;
}

void Proxy::setPort(std::string p) {
    this->port = p;
}

void Proxy::setUser(std::string u) {
    this->user = u;
}

void Proxy::setPassword(std::string p) {
    this->password = p;
}

void Proxy::setProxyFile(bool opt) {
        this->proxyFile = opt;
}

/*---------------------------------------------------------------------
 * get Functions
 */

std::string Proxy::getHost() const {
    return this->host;
}

std::string Proxy::getPort() const {
    return this->port;
}

std::string Proxy::getUser() const {
    return this->user;
}

std::string Proxy::getPassword() const {
    return this->password;
}

bool Proxy::proxyFileIsSet() const {
        return this->proxyFile;
}

/*---------------------------------------------------------------------
 * Function: setProxy
 * Summary:  If variables from proxy file setted,
 * then set the proxy object.
 */

bool set_proxy_object(Proxy* proxy, std::map<std::string,
                        std::string>* content) {
    if (content->count("PROXYHOST") == 0 ||
        content->count("PROXYPORT") == 0 ||
        content->count("PROXYUSER") == 0 ||
        content->count("PROXYPASSWORD") == 0)
            return false;

    proxy->setHost((*content)["PROXYHOST"]);
    proxy->setPort((*content)["PROXYPORT"]);
    proxy->setUser((*content)["PROXYUSER"]);
    proxy->setPassword((*content)["PROXYPASSWORD"]);

    return true;
}

/*---------------------------------------------------------------------
 * Function: parse_proxy_file
 * Summary: Read the proxy file and make the parse to proxy object.
 */

bool parse_proxy_file(Conf* conf, Proxy* proxy, Json::Value* data) {
    std::map<std::string, std::string> content;

    if (read_file(conf->getProxyConf(), &content)) {
        if (!set_proxy_object(proxy, &content)) {
            std::string var = __DATE__ +
                std::string(" - ERROR: to parse variables from proxy file.");

            (*data)["error"]["parse_variable_from_proxy_file"] =  var;
            return false;
        }
    }

    proxy->setProxyFile(true);
    return true;
}
