/* Copyright (C) 2016-2017 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of simmc-agent
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include <agent/main.h>
#include <agent/get.h>
#include <agent/post.h>
#include <string>

int run_agent(const sys::error_code& /*e*/, pt::ptime begin, bool print  ) {
    pt::ptime now;
    now = pt::second_clock::local_time();
    pt::time_duration t3 = now - begin;
    if ((t3.minutes() % 5 == 0 )) {
        send_net_bandwidth(print);
    }
    if ((t3.hours() % 24 == 0) && (t3.minutes() %60 == 0  )) {
        check_update();
        send_inventory(print);
        send_user_history(print);
    }
    return(0);
}

int set_begin(pt::ptime& begin, bool print) { // NOLINT (runtime/reference)
    std::string t;
    std::stringstream t2;
    std::fstream myfile("../build/time.txt",
         std::fstream::app | std::fstream::in | std::fstream::out);
    getline(myfile, t);
    if (t.size() == 0) {
        begin = pt::second_clock::local_time();
        myfile << begin;
        myfile << "\n";

        check_update();
        send_inventory(print);
        send_net_bandwidth(print);
        send_user_history(print);

        myfile.close();
        return(0);
    } else {
        t2 << t;
        t2 >> begin;

        myfile.close();
        return(1);
    }
}

int parse_command_line(int ac, char* av[], int *increment, int *lim,
     bool *print) {
    po::options_description desc("Allowed options");
    desc.add_options()
        ("help", "produce help message")
        ("once", "run agent only once")
        ("print", "set print JSON to true")
        ("uninstall", "uninstall agent and all associated files");

    po::variables_map vm;
    po::store(po::parse_command_line(ac, av, desc), vm);
    po::notify(vm);

    if (vm.count("help")) {
        std::cout << desc << "\n";
        return(1);
    }
    if (vm.count("once")) {
        *lim = 0;
    }
    if (vm.count("print")) {
        *print = true;
    } else {
        *print = false;
    }

    if (vm.count("uninstall")) {
        fs::remove_all("../../agent");
        return (1);
    }
    return(0);
}

int main(int argc, char* argv[]) {
    int increment = 0;
    int lim = 1;
    bool print;
    if (parse_command_line(argc, argv, &increment, &lim, &print)) {
        return (1);
    }
    pt::ptime begin;

    if (set_begin(begin, print) && lim == 0) {
        check_update();
        send_inventory(print);
        send_net_bandwidth(print);
        send_user_history(print);
        return (0);
    }
    int i = 0;
    while (i < lim) {
        asio::io_service io;
        asio::deadline_timer t(io, pt::seconds(60));
        t.async_wait(boost::bind(run_agent, pholders::error, begin, print));
        io.run();
        i += increment;
    }

    return(0);
}
