/* Copyright (C) 2017 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of simmc-agent
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include <agent/rotate_logfile.h>
#include <string>

/*---------------------------------------------------------------------
 * Method: setLogDir
 * Summary: Set the path of log dir.
 */

void Rotate::setLogDir(bf::path d) {
    this->logDir = d;
}

/*---------------------------------------------------------------------
 * Method: setNumberOfFiles
 * Summary: Count number of files that are in a specific directory.
 */
void Rotate::setNumberOfFiles() {
    int count = 0;
    for (bf::directory_iterator it(this->logDir); it != bf::directory_iterator(); ++it) // NOLINT [whitespace/line_length]
        count++;
    this->numberOfFiles = count;
}

/*---------------------------------------------------------------------
 * Method: setSizeOfLogfile
 * Summary: Gett the size of a specific file and set it.
 */
void Rotate::setSizeOfLogfile(Agent* agent) {
    std::fstream file((agent->getLogFile()).c_str(), std::fstream::app);
    this->sizeOfLogfile = file.tellg(); // NOLINT [runtime/int]
    file.close();
}

/*---------------------------------------------------------------------
 * Method: setLogfileName
 * Summary: Set tha name of logfile
 */
void Rotate::setLogfileName(std::string n) {
    this->logfileName = n;
}

/*---------------------------------------------------------------------
 * get Functions
 */
bf::path Rotate::getLogDir() const {
    return this->logDir;
}

int Rotate::getNumberOfFiles() const {
    return this->numberOfFiles;
}

std::string Rotate::getLogfileName() const {
    return this->logfileName;
}

long Rotate::getSizeOfLogfile() const { // NOLINT [runtime/int]
    return this->sizeOfLogfile;
}

/*---------------------------------------------------------------------
 * Method: logDirIsSet
 * Summary: CHeck if the path of logfile was setted.
 */
bool Rotate::logDirIsSet() {
    if (this->logDir.size() != 0)
        return true;
    return false;
}

/*---------------------------------------------------------------------
 * Method: createLogFile
 * Summary: Case rotated log file, is needed create a new logfile.
 */ 
void Rotate::createLogfile(Agent* agent, Json::Value* data) {
    std::fstream file(this->getLogfileName().c_str(), std::fstream::app);
    if (file.is_open()) {
        agent->setMessage(__DATE__ + std::string(" - SUCCESS: ") +
                    std::string(" Rotate logfile."));

        file << agent->getMessage() << std::endl;
        (*data)["success"]["rotate_logfile"] = agent->getMessage();
        file.close();
    } else {
        agent->setMessage(__DATE__ + std::string(" - ERROR: ") +
                    std::string(" Logfile cannot be created."));
        (*data)["error"]["file_not_created"] = agent->getMessage();
    }
}

/*---------------------------------------------------------------------
 * Method: renameFile
 * Summary: Rename log files.
 * Example:
    * Files inside log directory
        * datasid-agent.2.log, datasid-agent.3.log, datasid-agent.4.log, datasid-agent.log
    * datasid-agent.2.log renamed to datasid-agent.1.log
    * datasid-agent.3.log renamed to datasid-agent.2.log
    * datasid-agent.4.log renamed to datasid-agent.3.log
 */
void Rotate::renameFile(Agent* agent) {
    this->setNumberOfFiles();
    int count = 1, number = this->getNumberOfFiles();
    for (int i = count; i < number; i++)
        bf::rename((agent->getPrefix() +
            std::string("/log/datasid-agent.") +
            std::string(std::to_string(i+1)) + std::string(".log")),
            (agent->getPrefix() +
            std::string("/log/datasid-agent.") +
            std::string(std::to_string(count++)) + std::string(".log")));
}

/*---------------------------------------------------------------------
 * Function: rotate_log_file
 * Summary: Check the number of log files that are in log directory.
 * Case if the number more than 12 files, remove datasid-agent.1.log
 * After that, rename the files name in log directory.
 * Check also the size of datasid-agent.log file. 
 * Case if size more than 100.000 bytes, than create a new file.
 */
void rotate_log_file(Agent* agent, Json::Value* data) {
    Rotate rotate;
    rotate.setLogfileName(agent->getLogFile());
    rotate.setSizeOfLogfile(agent);
    rotate.setLogDir(agent->getPrefix() + std::string("/log"));
    rotate.setNumberOfFiles();
    rotate.getNumberOfFiles();

    if (rotate.logDirIsSet()) {
        if (rotate.getNumberOfFiles() < 5) {
            // Size - size of logfile in bytes.
            if (rotate.getSizeOfLogfile() > 5000000) {
                bf::rename(rotate.getLogfileName().c_str(), agent->getPrefix() + // NOLINT [whitespace/line_length]
                    std::string("/log/datasid-agent.") +
                    std::string(std::to_string(rotate.getNumberOfFiles())) +
                    std::string(".log"));
                rotate.createLogfile(agent, data);
            }
        } else {
            /*
             * Case if are more than 11 files inside log dir, 
             * remove datasid-agent.1.log file.
             */
            std::fstream logFile(rotate.getLogfileName().c_str(), std::fstream::app); // NOLINT [whitespace/line_length]
            if (logFile.is_open()) {
                if (remove((agent->getPrefix() +
                    std::string("/log/datasid-agent.1.log")).c_str()) != 0) {
                    agent->setMessage(__DATE__ + std::string(" - ERROR: ")
                        + std::string("Failed to remove logFile at rotate")
                        + std::string(" logfile function."));

                    logFile << agent->getMessage() << std::endl;
                    (*data)["error"]["failed_remove_logfile"] = agent->getMessage(); // NOLINT [whitespace/line_length]
                } else {
                    agent->setMessage(__DATE__ + std::string(" - SUCCESS: ") +
                                    std::string("Logfile removed."));

                    logFile << agent->getMessage() << std::endl;
                    (*data)["success"]["logfile_removeD"] = agent->getMessage();

                    agent->setMessage(__DATE__ + std::string(" - SUCCESS: ") +
                        std::string(" Rotate logfile."));

                    logFile << agent->getMessage() << std::endl;
                    (*data)["success"]["rotate_logfile"] = agent->getMessage();

                    rotate.renameFile(agent);
                }
                logFile.close();
            }
        }
    }
}
